#!/usr/bin/perl -w -T
#
# Library of procedures for use 
# by the EduRoam cgi-bin pages
#

package eduroam;

BEGIN { }

use strict;
use Net::SNMP;

##########################################
# Declaration of NET parameters
# - ebtables
# - arptables
# - iptables
# - network
# - network-scripts/ifcfg-*
# - /etc/resolv.conf

my $default_wan_interface = "";
my $wan_ip = "";
my $gw_ip = "";
my $dns = "";
my $hostname = "host.domain.tld";
my $domain_name = "domain.tld";
my $type = "";
my $user_vlan = "200";
my $man_iface_ip = "10.0.100.0/24";
my $user_iface_ip = "10.0.200.0/24";
my $management_network = "";
# This one is derived from other data
my @wlan_nets;

# AAI
my $realm_name = "";
my $ldap_root_password = "";
my $primary_radius_ip = "";
my $primary_radius_sec = "";
my $secondary_radius_ip = "";
my $secondary_radius_sec = "";
my @ldap_acls = ();
my %ldap_userdb;
my %radius_userdb;
# These ones are derived from other data
my $dc_domain;
my $root_dn;
my $root_pw;
my $domain;
my $user;
my $user_password;

# ACC
my $radiusd_pw = "";
my $dhcpd_pw = "";
my $apd_pw = "";

# AP
# aps is a complex record with the following attributes (in a hash):
# 1. hash key   - IP address of the Access Point
# 2. root_pw    - root password (username is root@<ip>)
# 3. snmp       - ro SNMP community
# 4. radio_mac  - Space seperated line of all radio MAC addresses (acquired via SNMP)
# 5. radius_sec - Radius secret for authentication
#
my %aps = ();

##########################################
# variable initialisation
sub BEGIN {
	# NET
	$eduroam::default_wan_interface = "";
	$eduroam::wan_ip = "";
	$eduroam::gw_ip = "";
	$eduroam::dns = "";
	$eduroam::hostname = "host.domain.tld";
	$eduroam::domain_name = "domain.tld";
	$eduroam::type = "";
	$eduroam::user_vlan = "200";
	$eduroam::man_iface_ip = "10.0.100.0/24";
	$eduroam::user_iface_ip = "10.0.200.0/24";
	$eduroam::management_network = "";

	# AAI
	$eduroam::realm_name = "";
	$eduroam::ldap_root_password = "";
	$eduroam::primary_radius_ip = "";
	$eduroam::primary_radius_sec = "";
	$eduroam::secondary_radius_ip = "";
	$eduroam::secondary_radius_sec = "";
	@eduroam::ldap_acls = ();
	
	%eduroam::aps = ();
}

##########################################
#########################################
# Substitutes variable in a line from a template file
# Returns an array of processed (values substituted) lines.
sub process_template_line {
        my @output = ();
        my $line = $_[0];
        my $tmp;

        # Add the line to the output only if there are no variables
        if (not $line =~ m/(^\$[^\$]|[^\$]\$[^\$]|[^\$]\$$)/) {
		$line =~ s/\${2}/\$/g; # handle escape sequence $$->$
		push @output, $line;
		return @output;
	}

	##########################################################################
	# NET componentes

        # $wlan_users.iface.vlan
        # -- all user interfaces and VLANs
        if ($line =~ m/\$wlan_users.iface.vlan/) {
                my $iface;
                for $iface (&get_phy_interfaces()) {
                        next if ("$iface" eq "$eduroam::default_wan_interface");
                        $tmp = $line;
                        $tmp =~ s/\$wlan_users.iface.vlan/$iface.$eduroam::user_vlan/;
                        push @output, &process_template_line($tmp);
                }
                return @output;
        }
                
        # $iface
        # -- all interfaces
        if ($line =~ m/\$iface/) {
                my $iface;
                for $iface (&get_phy_interfaces()) {
                        $tmp = $line;
                        $tmp =~ s/\$iface/$iface/;
                        push @output, &process_template_line($tmp);
                }
                return @output;
        }

        # $vlan
        # -- all user vlans (only 1 right now)
        if ($line =~ m/\$vlan/) {
                $tmp = $line;
                $tmp =~ s/\$vlan/$eduroam::user_vlan/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $management_network
        # -- ACLs for access to EduRoam server
        if ($line =~ m/\$management_network/) {
                my $acl;
                for $acl (split " ",$eduroam::management_network) {
                        $tmp = $line;
                        $tmp =~ s/\$management_network/$acl/;
                        push @output, &process_template_line($tmp);
                }
                return @output;
        }

        # $wlan_users_network
        # -- ACLs for new user connections
        # currently only one.
        if ($line =~ m/\$wlan_users_network/) {
                $tmp = $line;
                $tmp =~ s/\$wlan_users_network/$eduroam::user_iface_ip/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $wlan_management_network
        # -- ACLs for new connections from infrastructure (APs, switches, ...)
        if ($line =~ m/\$wlan_management_network/) {
                $tmp = $line;
                if (defined $eduroam::man_iface_ip and length($eduroam::man_iface_ip)>0) {
                        $tmp =~ s/\$wlan_management_network/$eduroam::man_iface_ip/;
                } else {
                        $tmp =~ s/\$wlan_management_network/$eduroam::wan_ip/;
                }
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $wlan_user
        # -- matching of VLAN and subnet 
        # -- used in ipfilters for anti-spoofing protection
        # -- specify all networks and all iface matches.
        if ($line =~ m/\$wlan_user\[net\]/ and $line =~ m/\$wlan_user\[iface\]/) {
                my $net;
                foreach $net (@eduroam::wlan_nets) {
                        $tmp = $line;
                        $tmp =~ s/\$wlan_user\[net\]/$net->[0]/;
                        $tmp =~ s/\$wlan_user\[iface\]/$net->[1]/;
                        push @output, &process_template_line($tmp);
                }
        }
	##########################################################################
        
	##########################################################################
	# AAI componentes
	
	# $ldap_management
        # -- ACLs for new connections from hosts that MAY access LDAP
        if ($line =~ m/\$ldap_management/) {
		my $ldap_src;
		foreach $ldap_src (@eduroam::ldap_acls) {
			$tmp = $line;
			$tmp =~ s/\$ldap_management/$ldap_src/;
			push @output, &process_template_line($tmp);
		}
                return @output;
        }
		
	# $radius_server
        # -- ACLs for new connections from other radius servers
        if ($line =~ m/\$radius_server/) {
		if(length($eduroam::primary_radius_ip) > 0) {
			$tmp = $line;
			$tmp =~ s/\$radius_server/$eduroam::primary_radius_ip/;
			push @output, &process_template_line($tmp);
		}
		if(length($eduroam::secondary_radius_ip) > 0) {
			$tmp = $line;
			$tmp =~ s/\$radius_server/$eduroam::secondary_radius_ip/;
			push @output, &process_template_line($tmp);
		}
                return @output;
        }

        # $dc_domain
        if ($line =~ m/\$dc_domain/) {
                $tmp = $line;
                $tmp =~ s/\$dc_domain/$eduroam::dc_domain/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $root_dn
        if ($line =~ m/\$root_dn/) {
                $tmp = $line;
                $tmp =~ s/\$root_dn/$eduroam::root_dn/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $root_pw
        if ($line =~ m/\$root_pw/) {
                $tmp = $line;
                $tmp =~ s/\$root_pw/$eduroam::root_pw/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $domain
        if ($line =~ m/\$domain/) {
                $tmp = $line;
                $tmp =~ s/\$domain/$eduroam::domain/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $full_domain
        if ($line =~ m/\$full_domain/) {
                $tmp = $line;
                $tmp =~ s/\$full_domain/$eduroam::realm_name/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $user_password
        if ($line =~ m/\$user_password/) {
                $tmp = $line;
                $tmp =~ s/\$user_password/$eduroam::user_password/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $user
        if ($line =~ m/\$user/) {
                $tmp = $line;
                $tmp =~ s/\$user/$eduroam::user/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        # $rad_user[user] in $rad_user[pass]
        # -- matching of radius username and password 
        if ($line =~ m/\$rad_user\[user\]/ and $line =~ m/\$rad_user\[pass\]/) {
                my $rad_user;
                foreach $rad_user (keys %eduroam::radius_userdb) {
                        $tmp = $line;
                        $tmp =~ s/\$rad_user\[user\]/$rad_user/;
                        $tmp =~ s/\$rad_user\[pass\]/$eduroam::radius_userdb{$rad_user}/;
                        push @output, &process_template_line($tmp);
                }
        }
	##########################################################################

	##########################################################################
	# ACC componentes

	# $radiusd_password
        if ($line =~ m/\$radiusd_password/) {
                $tmp = $line;
                $tmp =~ s/\$radiusd_password/$eduroam::radiusd_pw/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        if ($line =~ m/\$dhcpd_password/) {
                $tmp = $line;
                $tmp =~ s/\$dhcpd_password/$eduroam::dhcpd_pw/;
                push @output, &process_template_line($tmp);
                return @output;
        }

        if ($line =~ m/\$apd_password/) {
                $tmp = $line;
                $tmp =~ s/\$apd_password/$eduroam::apd_pw/;
                push @output, &process_template_line($tmp);
                return @output;
        }
	##########################################################################

	##########################################################################
	# AP componentes
        if ($line =~ m/\$ap/) {
		my $ap;
		foreach $ap (keys %eduroam::aps) {
                	$tmp = $line;
	                $tmp =~ s/\$ap/$ap/;
	                push @output, &process_template_line($tmp);
		}
                return @output;
        }
	##########################################################################

        return @output;
}
##########################################
sub process_aai {
	$eduroam::dc_domain = $eduroam::realm_name;
	$eduroam::dc_domain =~ s/\./,dc=/g;
	$eduroam::dc_domain =~ s/^/dc=/g;
	$eduroam::realm_name =~ m/^(\w+)\./;
	$eduroam::domain = $1;
	$eduroam::root_dn = "cn=root,$eduroam::dc_domain";
	$eduroam::root_pw = "$eduroam::ldap_root_password";
}
##########################################
sub process_wlan_nets {
	my $iface = "";
	my $tmp;

	foreach $iface (&get_phy_interfaces()) {
        	my $tmp;
	        next if ("$iface" eq "$default_wan_interface");
	        if (defined $man_iface_ip and length($man_iface_ip)>0) {
	                $tmp = ["$man_iface_ip", "$iface"];
	        } else {
	                $tmp = ["$wan_ip", "$iface"];
	        }
	        push @wlan_nets, $tmp;
	        $tmp = ["$user_iface_ip", "$iface.$user_vlan"];
	        push @wlan_nets, $tmp;
	}
}

##########################################
# Output /etc/raddb/clients.conf
#
sub establish_radius_clients {
	my $tmp;
	if (not open(FH, ">/etc/raddb/clients.conf")) {
	        print "ERROR: Unable to insert data to /etc/raddb/clients.conf";
	        exit -1;
	}

	# add upstrem radius servers
	print FH "# ===============================\n";
	print FH "# Upstream radius servers\n";
	print FH "# ===============================\n";
	if ("$eduroam::primary_radius_ip" ne "") {
	        print FH "client $eduroam::primary_radius_ip {\n";
	        print FH "\tsecret    = $eduroam::primary_radius_sec\n";
	        print FH "\tshortname = primary_radius\n";
	        print FH "\tnastype   = other\n";
	        print FH "}\n";
	}
	if ("$eduroam::secondary_radius_ip" ne "") {
	        print FH "client $eduroam::secondary_radius_ip {\n";
	        print FH "\tsecret    = $eduroam::secondary_radius_sec\n";
	        print FH "\tshortname = secondary_radius\n";
	        print FH "\tnastype   = other\n";
	        print FH "}\n";
	}
	print FH "# ===============================\n";
	print FH "# Access Points\n";
	print FH "# ===============================\n";
	foreach $tmp (keys %eduroam::aps) {
	        print FH "client " . $tmp . " {\n";
	        print FH "\tsecret    = " . $eduroam::aps{$tmp}->{"radius_sec"} . "\n";
	        # TODO: we should also store the type and short name (hostname) of AP
	        print FH "\tshortname = apX\n";
	        print FH "\tnastype   = cisco\n";
	        print FH "}\n";
	}
	print FH "# ===============================\n";
	close FH;
}

##########################################
# Output /etc/sysconfig/iptables
# and restart the firewall
#
# Warning! The config file must be already loaded!
#
sub establish_iptables {
	my @output = ();
	my $tmp;

	open(FH, "</var/eduroam/system_templates/iptables.eduroam");
	while(<FH>) {
	        push @output, &process_template_line($_);
	}
	close(FH);

	open(FH, ">/etc/sysconfig/iptables");
	print FH @output;
	close(FH);

	$tmp = `/sbin/service iptables restart`;
	print $tmp;
}
##########################################
sub load_conf {
        # The form is loaded the first time.    
        # load settings -- the ugly/perl way.
        my $config_file = $_[0];

        return "" if (! -f "$config_file");
        open FH, "<$config_file";
        while(<FH>) {
                next if(m/^#/);
                m/(.*)/;
                eval $1;
        }
}       

##########################################
my $apRadioMACTable = ".1.3.6.1.4.1.9.9.413.1.1.4.1.2.1";
sub get_ap_macs {
	my $ap = $_[0];
	my $community = $_[1];
	my $key;
	my $result;
	my $macs = "";
	my ($session, $error) = Net::SNMP->session(
		-hostname  => $ap,
		-community => $community,
		-translate => [ -octetstring => 0x0 ],
		-version   => "snmpv2c"
	);
                
	return undef if(!defined($session));
	
           
	$result = $session->get_table(
		-baseoid => $apRadioMACTable
	);

	return undef if(!defined($result));
	$session->close;

	# Process returned MAC from AP.
	foreach $key (keys(%{$result})) {
		my $mac = sprintf("%s.%s.%s", unpack("H4H4H4", $result->{$key}));
		$macs .= "$mac ";
        }
	$macs =~ s/\s+$//;
	return $macs;
}
##########################################
# Returns an array of ethX devices on the
# system.
sub get_phy_interfaces {
	my @ifaces;

	open(FD, "</proc/net/dev");
	while(<FD>) {
		if (m/^\s+(eth.*):/) {
			push(@ifaces, "$1");
		}
	}
	return @ifaces;
}

##########################################
# IpStr2Int
sub IpStr2Int {
	return undef unless ($_[0] =~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/);
	return ($1<<24 | $2<<16 | $3<<8 | $4) & 0xFFFFFFFF;
}
##########################################
# IpInt2Str
sub IpInt2Str {
	return sprintf("%d.%d.%d.%d", $_[0]>>24&0xFF, $_[0]>>16&0xFF, $_[0]>>8&0xFF, $_[0]&0xFF);
}
##########################################
# Returns netmask from IP/CIDR
sub IpCidr2IpNetmask {
	my $cidr;
	my $ip;
        my $str_net = $_[0];
	my $bit_mask;
	my $nmask;
	return undef unless ($str_net =~ m/^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,2})$/);
	$ip = $1;
	$cidr = 0+$2;
	return undef unless ($cidr>=0 && $cidr<=32);
	$bit_mask  = (0xFFFFFFFF<<(32-$cidr));
	$bit_mask &=  0xFFFFFFFF;
	$nmask = &IpInt2Str($bit_mask);
	return ($ip, $nmask);
}
##########################################
# Return min and max usable address
sub IpCidr2MinMaxIP {
	my $ipmin;
	my $ipmax;
	my $cidr;
	my $ip;
	my $bit_mask;
        my $str_net = $_[0];

	return undef unless ($str_net =~ m/^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,2})$/);
	$ip = &IpStr2Int($1);
	return undef if (!defined $ip);
	$cidr = 0+$2;
	return undef unless ($cidr>=0 && $cidr<=32);
	$bit_mask  = (0xFFFFFFFF<<(32-$cidr));
	$bit_mask &=  0xFFFFFFFF;
	$ipmin     =  ($ip &  $bit_mask) +1;
	$ipmax     =  ($ip | ~$bit_mask) -1;
	$ipmin     = &IpInt2Str($ipmin);
	$ipmax     = &IpInt2Str($ipmax);
	return ($ipmin, $ipmax);
}

##########################################
# Executes a command or prints out a HTML error message.
sub system_or_htmldie {
	my $message;
	my $error = $_[1];
	my $tmp;
	my $i = 0;
	
	$error = "Failed to execute $_[0]" if !defined $_[1];
	STDOUT->autoflush(1);
	&html_die($error) if (not open (FH, "$_[0] 2>&1 |"));
	FH->autoflush(1);
	while(0 != sysread FH, $tmp, 1) {
		if("$tmp" eq "\n") {
			$i = 0;
			print "<br>\n";
			next;
		}
		print $tmp;
		$i++;
		print "<br>\n" if (0==$i%100);
	}
	close FH;
}
#sub system_or_htmldie {
#	my $message;
#	my $error = $_[1];
#                        
#	$error = "Failed to execute $_[0]" if !defined $_[1];
#                
#	$message = `$_[0] 2>&1`;
#	if ($?) {
#		print "Content-Type: text/html\n\n";
#		print "ERROR: $error<br>\n";
#		if (defined $message and length $message > 0) {
#			$message =~ s/\n/<br>/g;
#			print "Detailed error report:<br>$message";
#		}
#		exit -1; 
#	}       
#	return $message;
#}               
##########################################
# Report the error passed as the first parameter and die.
# Second (optional) parameter is a web form to return to.
# Third (optional) parameter is the end HTML.
# ----------------------------------------
# Content-Type and maybe some header must
# have been already printed out.
sub html_die {
	print "Critical ERROR: $_[0]<BR>\n";
	print "Check web server log files.\n";
	if(defined $_[1]) {
		print "<FORM METHOD=POST ACTION=\"$_[1]\">";
		print "<INPUT TYPE=\"submit\" NAME=Confirm VALUE=Confirm><br>";
		print "</FORM>";
	}
	print $_[2] if (defined $_[2]);
	exit -1;
} 
##########################################
sub html_perror {
	print "Content-Type: text/html\n\n";
	print "$_[0]<BR>\n";
	print "<FORM METHOD=POST ACTION=\"$_[1]\">";
	print "<INPUT TYPE=\"submit\" NAME=Confirm VALUE=Confirm><br>";
	print "</FORM>";
	exit 0;
}
##########################################

return 1;

END {}
