#!/usr/bin/perl 
#
# Server's tasks:
# - tail dhcp log and log complement logs in MySQL with IP information
# - check active/stale user connections on APs and close old accounting records (stale connections)
# - Manages L2 firewall thru ebtables and arptables
#

use strict;
use File::Tail;
use Time::Local;
use Sys::Syslog;
use POSIX;
use DBI;
use Net::SNMP;
use Fcntl ':flock';

############################################
# Configuration                            # 
############################################
my $dsn = 'DBI:mysql:radius:localhost';
my $sql_user_name_dhcp = 'dhcpd';
my $sql_user_name_apd = 'apd';
my $logfile = '/var/log/dhcpd.log';
my $fw_lockfile = "/var/lock/eduroam/LCK..firewall";

# Which facility should messages be logged to
my $facility = 'local6'; # write log to the facility X.
my $check_conn = 10*60; # Check existing connection every X seconds: 10 minutes.
my $online_time_check = 60*60*24; # 24 hrs
my $lock_timeout = 30; # How long should we try to update the firewall: 30 seconds.
my $adjust_l2_firewall = 1; # Adjust the firewall on the server ?

# Load "libraries"
require '/usr/lib/eduroam/eduroam.pm';

my $config_file1 = "/var/eduroam/etc/ap.conf";
my $config_file2 = "/var/eduroam/etc/acc.conf";
my $config_file3 = "/var/eduroam/etc/net.conf";

my $ebtables_tool = "/usr/sbin/ebtables";
# ==========================================
# and constants/variables
my $apClientTable = ".1.3.6.1.4.1.9.9.273.1.2.1.1.14";
my $apRadioMACTable = ".1.3.6.1.4.1.9.9.413.1.1.4.1.2.1";
my %months = ('Jan', 0, 'Feb', 1, 'Mar', 2, 'Apr', 3, 'May',  4, 'Jun',  5,
	      'Jul', 6, 'Aug', 7, 'Sep', 8, 'Oct', 9, 'Nov', 10, 'Dec', 11);
my $pid_dhcp = 0;
my $pid_ap = 0;
my $logid = "M:";
my $file;
my $line;
my $rc;
my $dbh;
my $sql_dhcp;
my $sql_get;
my $sql_close;
my %ap_mac_ip = ();

# ==========================================
sub signal_handler {
    syslog('err', "$logid Received signal $_[0], TERMINATING!");
    exit(-1);
}
# ==========================================
# convert_cisco_mac - convert MAC from cisco format
#
sub convert_cisco_mac {
        if ($_[0] =~ m/([0-9,A-F,a-f]{2})([0-9,A-F,a-f]{2})\.([0-9,A-F,a-f]{2})([0-9,A-F,a-f]{2})\.([0-9,A-F,a-f]{2})([0-9,A-F,a-f]{2})/) {
                return sprintf("%02x:%02x:%02x:%02x:%02x:%02x", hex $1, hex $2, hex $3, hex $4, hex $5, hex $6);
        }
        return $_[0];
}
# ==========================================
# slim_mac - convert MAC to slim format
#
sub slim_mac {
        my $mac = &convert_cisco_mac($_[0]);
        unless ($mac =~ m/^([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})$/)
		{ return undef };
        return sprintf("%x:%x:%x:%x:%x:%x", hex $1, hex $2, hex $3, hex $4, hex $5, hex $6);
}
# ==========================================
# fat_mac - convert MAC to fat format
#
sub fat_mac {
        my $mac = &convert_cisco_mac($_[0]);
        unless ($mac =~ m/^([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})[.:-]([0-9,A-F,a-f]{1,2})$/)
		{ return undef };
        return sprintf("%02x:%02x:%02x:%02x:%02x:%02x", hex $1, hex $2, hex $3, hex $4, hex $5, hex $6);
}
# ==========================================
sub lock_firewall {
        unless (open FH, "+< $fw_lockfile") {
    		syslog('err', "$logid Unable to open firewall lock-file: $fw_lockfile. TERMINATING!");
		return 0;
	}
        for (my $time = $lock_timeout; $time > 0; $time --) {
                if (flock(FH, LOCK_EX | LOCK_NB)) {
                        return 1;
                }
                sleep(1);
        }
        return 0;
}
# ==========================================
sub unlock_firewall {
        close(FH);
}
# ==========================================
sub ebtables_get {
        my @lines;
        my $text = `$ebtables_tool -L USERS`;
        if (0 != $?) {
                syslog('err', "Unable to list ebtables. Is tool 'ebtables' installed ? TERMINATING!");
                exit(-1);
        }
        @lines = split("\n", $text);
        splice @lines, 0, 3;
        return @lines;
}
# ==========================================
sub arptables_get {
        my @lines;
        my $text = `/sbin/arptables -L USERS -n`;
        if (0 != $?) {
                syslog('err', "Unable to list arptables. Is tool 'arptables' installed ? TERMINATING!");
                exit(-1);
        }
        @lines = split("\n", $text);
        splice @lines, 0, 1;
        return @lines;
}
# ==========================================
sub fw_run_eblines {
        my $ebline;
        foreach $ebline (@{$_[0]}) {
		syslog('debug', "$logid ebtables update: $ebline");
                `$ebtables_tool $ebline 2>/dev/null`;
                if (0 != $?) {
                        syslog('err', "Unable to process ebtables. Is tool 'ebtables' installed ? TERMINATING!");
                        exit(-1);
                }
        }
}
# ==========================================
sub fw_run_arplines {
        my $arpline;
        foreach $arpline (@{$_[0]}) {
		syslog('debug', "$logid arptables update: $arpline");
                `/sbin/arptables $arpline 2>/dev/null`;
                if (0 != $?) {
                        syslog('err', "Unable to process arptables. Is tool 'arptables' installed ? TERMINATING!");
                        exit(-1);
                }
        }
}
# ==========================================
sub delete_firewall {
        my $ip = $_[0];
        my $ebmac = &slim_mac($_[1]);
        my $arpmac = &fat_mac($_[1]);
        my $line = "";
        my $text_rules = "";
        
        # Only update the firewall rules if instructed to do so.
        return if($adjust_l2_firewall != 1);
	if (! &lock_firewall()) {
		syslog('err', "$logid Unable to lock firewall for $ip($arpmac). TERMINATING!");
		exit(-1);
	}
        
        # process ebtables, find matching lines and delete them
        $text_rules = `$ebtables_tool -L USERS`; 
       	if (0 != $?) {
		syslog('err', "$logid Unable to list ebtables for $ip($arpmac). Is tool 'ebtable' installed ? TERMINATING!");
		exit(-1);
	}
        foreach $line (split("\n",$text_rules)) {
                unless ($line =~ m/$ebmac.*$ip/) { next };
                $line = `$ebtables_tool -D USERS $line`;
        	if (0 != $?) {
			syslog('err', "$logid Unable to delete ebtables for $ip($arpmac). TERMINATING!");
			exit(-1);
		}
        }

        # process arptables, find matching lines and delete them
        $text_rules = `/sbin/arptables -L USERS`; 
       	if (0 != $?) {
		syslog('err', "$logid Unable to list arptables for $ip($arpmac). Is tool 'arptable' installed ? TERMINATING!");
		exit(-1);
	}
        foreach $line (split("\n",$text_rules)) {
                unless ($line =~ m/$ip.*$arpmac/) { next };
                $line = `/sbin/arptables -D USERS $line 2>/dev/null`;
        	if (0 != $?) {
			syslog('err', "$logid Unable to delete arptables for $ip($arpmac). TERMINATING!");
			exit(-1);
		}
        }
	&unlock_firewall();
}
# ==========================================
sub update_firewall {
        my $ip = $_[0];
        my $mac = &fat_mac($_[1]);
        my $rc;

        # Only update the firewall rules if instructed to do so.
        return if($adjust_l2_firewall != 1);
	if (! &lock_firewall()) {
		syslog('err', "$logid Unable to lock firewall for $ip($mac). TERMINATING!");
		exit(-1);
	}
	
	# Adjust the firewall for the new client
	$rc = system("$ebtables_tool -A USERS -p IPv4 -s $mac --ip-source $ip -j ACCEPT");
        if (0 != $rc) {
		syslog('err', "$logid Unable to update ebtables for $ip($mac). TERMINATING!");
		exit(-1);
	}
	$rc = system("/sbin/arptables -A USERS -s $ip --source-mac $mac -j ACCEPT");
        if (0 != $rc) {
		syslog('err', "$logid Unable to update arptables for $ip($mac). TERMINATING!");
		exit(-1);
	}
	&unlock_firewall();
}
# ==========================================
# sql_users_get - fetches all active session
# records from the SQL database and returns
# them in a hash table %users.
# 
sub sql_users_get {
        my $tmp;
        my $key;
        my $data;
        my $result;
        my %users;      # Key is a MAC, data is array[IP,IP, ...]
	my $dbh;        # TODO: use existing connection to the database!!!

        $dbh = DBI->connect($dsn, $sql_user_name_apd, $eduroam::apd_pw);
	if (not defined $dbh) {
		syslog('err', "$logid ERROR in sql_users_get(), unable to connect to database");
		return undef;
	}

        $sql_get = $dbh->prepare(q{SELECT 
                `Calling-Station-Id`,
                `Client-IP-Address`
        FROM ACCOUNTING WHERE
                `Timestamp Stop` = FROM_UNIXTIME(0) 
        AND     `Client-IP-Address` != ''
        ORDER BY
                `Timestamp Start` DESC
        });

        $sql_get->execute() || die $dbh->errstr;
        while ($result = $sql_get->fetchrow_arrayref) {
                # The data returned is: Calling-Station-Id, Client-IP-Address
		syslog('debug', "$logid SQL active session: $result->[0] ($result->[1])");
                $tmp = $users{$result->[0]};
                push(@$tmp, $result->[1]);
                $users{$result->[0]} = $tmp;
        }

        $dbh->disconnect;
        return %users;
}
# ==========================================
# fw_sync synchronises the (arp/eb)tables, chain USERS
# with the data stored in the SQL database.
# Old fw entries are deleted, new are inserted if needed.
# So we actualy DO sync and not just flush chains and
# reinsert. Flushing and reinserting could take too
# long and users might loose connectivity during the fw
# reinsertion.
#
sub fw_sync {
        my $i;  
	my $ip;
	my $mac;
	my $line;
	my @eblines;
	my @add_eblines;
	my @arplines;
	my @add_arplines;
	my %sql_users;
        
	# Only update the firewall rules if instructed to do so.
        return if($adjust_l2_firewall != 1);
	if (! &lock_firewall()) {
		syslog('err', "$logid Unable to lock firewall. TERMINATING!");
		exit(-1);
	}

	@eblines = &ebtables_get;
	@arplines = &arptables_get;
	%sql_users = &sql_users_get;

	# All entries in fw will be deleted by default.
	for($i=0; $i<= $#eblines; $i++) {
		$eblines[$i] = "-D USERS $eblines[$i]";
	}
	for($i=0; $i<= $#arplines; $i++) {
		$arplines[$i] = "-D USERS $arplines[$i]";
	}

	foreach $mac (keys %sql_users) {
		syslog('err', "$logid Running $mac\n");
	        my $ebmac = &slim_mac($mac);
	        my $arpmac = &fat_mac($mac);
	        foreach $ip (@{$sql_users{$mac}}) {
                	for($i=0; $i<=$#eblines; $i++) {
                        	if ($eblines[$i] =~ m/$ebmac.*$ip/i) {
                                	splice(@eblines,$i--,1);
	                                last;
	                        }
	                }
	                # ip/mac is in DB but not in the fw/eb
	                if($i > $#eblines or 0 == $i) {
	                        push(@add_eblines, "-A USERS -p IPv4 -s $ebmac --ip-source $ip -j ACCEPT");
	                }
	                for($i=0; $i<=$#arplines; $i++) {
	                        if ($arplines[$i] =~ m/$ip.*$arpmac/i) {
	                                splice(@arplines,$i--,1);
	                                last;
	                        }
	                }
	                # ip/mac is in DB but not in the fw/arp
	                if($i > $#arplines or 0 == $i) {
	                        push(@add_arplines, "-A USERS -j ACCEPT -s $ip --source-mac $arpmac");
	                }
	        }
	}
	push(@eblines, @add_eblines);
	push(@arplines, @add_arplines);
       	&fw_run_eblines(\@eblines);
        &fw_run_arplines(\@arplines);
	&unlock_firewall;
}
# ==========================================
# Create daemon and setup syslog logging
sub daemonize {
    my $pid = 1;
    my $rc = 0;
    openlog('eduroam_monitor', 'ndelay,pid', $facility)
        or die "Can't open syslog: $!";
    chdir '/'                  or die "Can't chdir to /: $!";
    umask 0;
    open STDIN, '/dev/null'    or die "Can't read /dev/null: $!";
    setsid                     or die "Can't start a new session: $!";
    open STDOUT, '>>/dev/null' or die "Can't write to /dev/null: $!";
    open STDERR, '>>/dev/null' or die "Can't write to /dev/null: $!";
    $SIG{INT}='signal_handler';
    $SIG{TERM}='signal_handler';
    $SIG{KILL}='signal_handler';
    $SIG{__DIE__}='signal_handler';
    $pid = fork;
    defined($pid) or syslog('err', "$logid Can't fork: $!");
    exit 0 if $pid;
}
# ==========================================
sub connect_to_mysql_dhcp {
	$dbh = DBI->connect($dsn, $sql_user_name_dhcp, $eduroam::dhcpd_pw);
	if(!defined($dbh)) {
		syslog('err', "$logid ERROR: connect to MySQL database failed!\n");
		exit(-1);
	}
	
        $sql_dhcp = $dbh->prepare(q{UPDATE ACCOUNTING SET 
                `Client-IP-Address` = ?,
                `Timestamp Dhcp` = NOW()
        WHERE
                `Calling-Station-Id` = ? AND
                `Timestamp Start` <= FROM_UNIXTIME(?) AND
                `Timestamp Stop` = FROM_UNIXTIME(0) 
        });
}
# ==========================================
sub connect_to_mysql_ap {
	$dbh = DBI->connect($dsn, $sql_user_name_apd, $eduroam::apd_pw);
	if(!defined($dbh)) {
		syslog('err', "$logid ERROR: connect to MySQL database failed!\n");
		exit(-1);
	}

        $sql_get = $dbh->prepare(q{SELECT 
                `Calling-Station-Id`,
                `User-Name`,
                UNIX_TIMESTAMP(`Timestamp Start`),
                `Client-IP-Address`,
                `Called-Station-Id`
        FROM ACCOUNTING WHERE
                `Timestamp Stop` = FROM_UNIXTIME(0) 
        ORDER BY
                `Timestamp Start` DESC
        });

        $sql_close = $dbh->prepare(q{UPDATE ACCOUNTING SET
                `Timestamp Stop`         = NOW(),
                `Acct-Terminate-Cause`   = 'Script-Cleanup'
        WHERE   `Calling-Station-Id` = ?
        AND     `User-Name` = ?
        AND     `Client-IP-Address` = ?
        AND     `Called-Station-Id` = ?
        AND     `Timestamp Stop` = FROM_UNIXTIME(0) 
        AND     `Timestamp Start` = FROM_UNIXTIME(?)
        });
}
# ==========================================
# Check if client with a given MAC is online 
sub check_client {
        my $mac = $_[0];
        my $ap_mac = $_[1];
	my $ap_community;
        my $ap_ip;
        my $key;
        my $mac_oid;
        my $result;

        defined($ap_mac_ip{$ap_mac}) or die "IP address for NAS $ap_mac isn't defined\n";
        $ap_ip = $ap_mac_ip{$ap_mac};                           # Get AP IP
	$ap_community = $eduroam::aps{$ap_ip}->{'snmp'};   # Get AP radius shared secret

        $mac =~ m/(..)(..)\.(..)(..)\.(..)(..)/;
        $mac_oid = sprintf("%d.%d.%d.%d.%d.%d", hex $1, hex $2, hex $3, hex $4, hex $5, hex $6);
        my ($session, $error) = Net::SNMP->session(
                -hostname  => $ap_ip,
                -community => $ap_community,
                -translate => [ -octetstring => 0x0 ],
                -version   => "snmpv2c"
        );

        if(!defined($session)) {
                syslog('err', "$logid ERROR in SNMP session establish: %s\n", $error);
                return 1;
        }

        $result = $session->get_table(
                -baseoid => $apClientTable
        );
        if(!defined($result)) {
                if(!$session->error_index) {
                        return 0; # an empty table "error", that's OK.
                }
                # Complain and that's it. Return as if user is present.
                syslog('err',"$logid ERROR in SNMP get_table: %s\n", $session->error);
                return 1;
        }
        $session->close;

        foreach $key (keys(%{$result})) {
                if ($key =~ m/$mac_oid$/) {
                        return $result->{$key} == 3;
                }
        }
        return 0;
}
# ==========================================
# Monitor dhcp log file and insert IP address
sub monitor_dhcp {
	my $rc;

	&connect_to_mysql_dhcp;
	
	$file=File::Tail->new(name=>$logfile, maxinterval=>1, interval=>1);
	syslog('notice',"$logid DHCP monitor running, PID: $$\n");
	# Set initial state of the fw
	&fw_sync;
	
	while( 1 ) {
		$line = $file->read;
		if(!defined($line)) {
			syslog('err',"$logid Unable to get a line from DHCP log! TERMINATING!");
			exit(-1);
		}
		$rc = $dbh->ping;
		if(1 != $rc) {
			syslog('warning',"$logid Lost connection to MySQL database! Restarting the connection!");
			$dbh->disconnect;		
			&connect_to_mysql_dhcp;
			$rc = $dbh->ping;
			if(1 != $rc) {
				syslog('err',"$logid Again lost connection to MySQL database! TERMINATING!");
				exit(-1);
			}
		}
	
		if ( $line =~ m/^(\w+)\s+(\d+)\s+(\d+)\:(\d+)\:(\d+).*dhcpd\: DHCPACK.on.(\S+).to.(\S+).*via.(\S+)/ ) {
			my $month = $1;
			my $day = $2;
			my $h = $3;
			my $m = $4;
			my $s = $5;
			my $ip = $6;
			my $mac = $7;
			my $iface = $8;
			my $year = (localtime)[5] + 1900;

			# Check if this is a request for a "managed" VLAN/interface. 
			next unless ($iface =~ m/^\w+\.$eduroam::user_vlan$/);

			$month = $months{$month};
			# check year change!
			if( 11 == $month && 31 == $day ) {
				my $cur_day = (localtime)[3];
				# If it's january the 1st, we missed a year change!
				if( 1 == $cur_day) {
					$year--;
				}
			}
			# change MAC from PC to Cisco AP format -- can also be changed in AP config
			$mac =~ s/(..)\:(..)\:(..)\:(..)\:(..)\:(..)/$1$2\.$3$4\.$5$6/;
			my $timestamp = timelocal($s, $m, $h, $day, $month, $year);
			$rc = $sql_dhcp->execute("$ip", "$mac", $timestamp ) || die $dbh->errstr;
			if ($rc < 1) {
				syslog('err', "$logid FATAL ERROR inserting DHCP assigned address: $year-%02d-%02d, %02d:%02d:%02d, IP: $ip, MAC: $mac, timestamp: $timestamp", $month + 1, $day, $h, $m, $s);
				exit -1;
			}
			syslog('info', "$logid Inserted DHCP assigned address: $year-%02d-%02d, %02d:%02d:%02d, IP: $ip, MAC: $mac, timestamp: $timestamp", $month + 1, $day, $h, $m, $s);
			#&update_firewall($ip, $mac);
			&fw_sync;
			syslog('info', "$logid Updated L2 firewall with IP: $ip, MAC: $mac");
			#$dbh->commit || die $dbh->errstr;
		}
	}
	$dbh->disconnect;
	exit(0);
}
# ==========================================
# Check connections on every interval
sub monitor_connections {
	my $rc;
	my $tmp;
	my $key;
	my $data;
	my $result;
	my %users;	# Key is a MAC, data is array[array[User, Start, IP, AP MAC]]

	syslog('notice',"$logid Access Point monitor running, PID: $$\n");
	&connect_to_mysql_ap;

	while (1) {
		%users = {};
		$rc = $dbh->ping;
		if(1 != $rc) {
			syslog('warning',"$logid Lost connection to MySQL database! Restarting the connection!");
			$dbh->disconnect;		
			&connect_to_mysql_dhcp;
			$rc = $dbh->ping;
			if(1 != $rc) {
				syslog('err',"$logid Again lost connection to MySQL database! TERMINATING!");
				exit(-1);
			}
		}

		$sql_get->execute() || die $dbh->errstr;
		while ($result = $sql_get->fetchrow_arrayref) {
        		# The data returned is: Calling-Station-Id, User-Name, Timestamp Start, Client-IP-Address, Called-Station-Id
	        	syslog('debug', "$logid User $result->[1], mac: $result->[0] on ap_list{$result->[4]}, Start: $result->[2]\n");
	        	$tmp = $users{$result->[0]}; # we want to append a record to an already existing key.
	        	push(@$tmp, [$result->[1], $result->[2], $result->[3], $result->[4]]);
	        	$users{$result->[0]} = $tmp;
		}               

		foreach $key (keys(%users)) {
        		my $i = 0;
        		$tmp = $users{$key};
	        	for $data (@$tmp) {
        	        	syslog('debug', "$logid MAC($i): $key, USER: $data->[0], Start: $data->[1], IP: $data->[2], AP MAC: $data->[3]\n");
	        	        if(!$i and $data->[1] + $online_time_check < time()) {
					syslog('debug',"$logid Record IS OLD, checking user\n");
					if(!check_client($key, $data->[3])) {
                                		syslog('debug', "$logid User not present, closing the accounting record\n");
	                                	$sql_close->execute($key, $data->[0], $data->[2], $data->[3], $data->[1]) || die $dbh->errstr;
        	        			syslog('info', "$logid Accounting closed. MAC: $key, USER: $data->[0], Start: $data->[1], IP: $data->[2], AP MAC: $data->[3]\n");
        	                	} else {
	                                	syslog('debug', "$logid User is still connected to the AccessPoint\n");
					}
                		}
                		if ($i) {
                        		syslog('debug', "$logid Record is a duplicate, checking user\n");
                        		if(!check_client($key, $data->[3])) {
	                                	syslog('debug', "$logid User not present, closing the accounting record\n");
        	                        	$sql_close->execute($key, $data->[0], $data->[2], $data->[3], $data->[1]) || die $dbh->errstr;
        	        			syslog('info', "$logid Accounting closed. MAC: $key, USER: $data->[0], Start: $data->[1], IP: $data->[2], AP MAC: $data->[3]\n");
                	        	} else {
	                                	syslog('debug', "$logid User is connected to the AccessPoint\n");
					}
                		}
			$i++;
        		}
		}
		sleep($check_conn);
	}

	$dbh->disconnect;
	exit(0);
}

# ==========================================
sub map_ap_mac_to_ip {
	my $ap;
	my $mac;
	my $macs;

	foreach $ap (keys %eduroam::aps) {
		$macs = $eduroam::aps{$ap}->{'mac'};
		foreach $mac (split ' ', $macs) {
			$ap_mac_ip{$mac} = $ap;
			syslog('debug', "$logid MAC $mac is mapped to ip $ap");
		}
	}
}
# ==========================================

# Load settings
&eduroam::load_conf($config_file1);
&eduroam::load_conf($config_file2);
&eduroam::load_conf($config_file3);

# Fork as a daemon and set-up logging to syslog.
&daemonize;

# Process loadeddata
# Create hash table with Radio MAC -> IP mapping
&map_ap_mac_to_ip;

while(1) {
	# If child doesn't exist, create it.
	if(!$pid_dhcp) {
		$pid_dhcp = fork;
		defined($pid_dhcp) or syslog('err', "$logid FATAL: Can't fork: $!");
		if(!$pid_dhcp) {
			$logid = "D:";
			monitor_dhcp();	
		}
	}
	if(!$pid_ap) {
		$pid_ap = fork;
		defined($pid_ap) or syslog('err', "$logid FATAL: Can't fork: $!");
		if(!$pid_ap) {
			$logid = "A:";
			monitor_connections();	
		}
	}

	$rc = wait();
        if($rc <= 0) {
		syslog('err', "$logid Error $rc, waiting for child");
		exit -1;
	}
	if ($rc == $pid_dhcp) {
		syslog('warning', "$logid DHCP child $pid_dhcp died, restarting it!");
		$pid_dhcp = 0;
		sleep(5); # So the process doesn't go wild && crazy
	}
	if ($rc == $pid_ap) {
		syslog('warning', "$logid AP child $pid_ap died, restarting it!");
		$pid_ap = 0;
		sleep(5); # So the process doesn't go wild && crazy
	}
}

exit 0;
